package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.TaCarRecordMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.TencentFaceUtils;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import com.tencentcloudapi.iai.v20180301.models.Candidate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-18
 */
@Service
public class TaCarRecordServiceImpl extends ServiceImpl<TaCarRecordMapper, TaCarRecord> implements ITaCarRecordService {
    Logger log = LoggerFactory.getLogger(TaCarRecordServiceImpl.class);

    @Autowired
    private TaCarRecordMapper taCarRecordMapper;

    @Autowired
    private ITaCarService iTaCarService;

    @Autowired
    private ITaPersonService iTaPersonService;

    @Autowired
    private ITaFirstUsherRecordService iTaFirstUsherRecordService;

    @Autowired
    private ITaCustomerService iTaCustomerService;

    @Autowired
    private TencentFaceUtils tencentFaceUtils;

    @Autowired
    private ISysParamService iSysParamService;

    @Autowired
    private ITaVisitorAppointmentService iTaVisitorAppointmentService;

    @Override
    public IPage<TaCarRecord> getPageList(IPage<TaCarRecord> page, String plateNumber, Date recordDate, String customerName, String phone) {
        return taCarRecordMapper.getPageList(page, plateNumber, recordDate, customerName, phone);
    }

    /**
     * 车辆抓拍之后
     * @param taCarRecord
     * @return
     */
    @Override
    public boolean newRecordByDevice(TaCarRecord taCarRecord) {
        // 是否内部车辆
        TaCar taCar = iTaCarService.getById(taCarRecord.getPlateNumber());
        if (null != taCar && taCar.getTypeId().equals(CommConstant.TYPE_CAR_INNER)) {
            return true;
        }

        // 到客户中查询是否有相关人员
        TaCustomer taCustomer = null;
        List<TaCustomer> taCustomerList = iTaCustomerService.getSomeCustMayBe(taCarRecord.getPlateNumber());

        // 如果没有对应人员
        if (null == taCustomerList || taCustomerList.size() == 0) {
            // 再去人脸库比对一次
            Integer personId = null;
            try {
                Float threshold = iSysParamService.getSamePersonThreshold();
                Candidate result = tencentFaceUtils.searchFace(taCarRecord.getDriverImage());
                if (result.getScore() >= threshold) {
                    personId = Integer.valueOf(result.getPersonId());
                }
            } catch (TencentCloudSDKException e) {
                e.printStackTrace();
            }

            // 依据人脸与预约判断
            // 人脸优先级高
            if (null != personId && personId > 0) {
                // 是否内部人员
                TaPerson taPerson = iTaPersonService.getByIdOrReal(personId);
                if (null != taPerson.getTypeId() && taPerson.getTypeId() == CommConstant.TYPE_INNER_PERSON) {
                    return true;
                }

                // 找到对应人脸
                taCustomer = iTaCustomerService.getSameByPerson(personId);
                if (null != taCustomer) {
                    // 尝试更新部分客户信息
                    iTaCustomerService.updataByCarRecord(taCustomer, taCarRecord);
                }
            }

            // 去预约记录查询一次
            List<TaVisitorAppointment> appointments = iTaVisitorAppointmentService.getAppointmentsByPlateNumber(taCarRecord.getPlateNumber());

            if (null == taCustomer) {
                if (null != appointments && appointments.size() > 0) {
                    taCustomer = iTaCustomerService.newCustByCarDevice(taCarRecord, appointments.get(0));
                    taCarRecord.setPhone(appointments.get(0).getPhone());
                    taCarRecord.setCustomerName(appointments.get(0).getPersonName());
                    taCarRecord.setPersonNum(appointments.get(0).getPersonNum());
                } else {
                    taCustomer = iTaCustomerService.newCustByCarDevice(taCarRecord);
                }
            }
        } else {
            // 对应客户, 先随机取一条
            taCustomer = taCustomerList.get(0);
        }

        // 先入库
        taCarRecordMapper.insert(taCarRecord);
        iTaCarService.newCarByDevice(taCarRecord);

        // 生成保安迎宾记录, 并通知小程序
        TaFirstUsherRecord taFirstUsherRecord = iTaFirstUsherRecordService.newRecordByDevice(taCarRecord);
        iTaFirstUsherRecordService.notifyMiniapp(taFirstUsherRecord);

        // 如果客户有固定置业, 推送消息给置业
        if (null != taCustomer.getConsultantId()) {
            try {
                iTaCustomerService.notifyConsultant(taCustomer);
            } catch (Exception e) {
                log.error("发送消息给置业失败: {}", e.getMessage());
            }
        }

        return true;
    }

    @Override
    public boolean updateRecordByMiniapp(TaCarRecord taCarRecord) {
        // 先暂时只做更新动作
        return taCarRecordMapper.updateById(taCarRecord) > 0;
    }

    @Override
    public List<TaCarRecord> getTimeline() {
        LocalDateTime now = LocalDateTime.now();

        // 四小时内
        LocalDateTime fourHour = now.minusHours(4);
        QueryWrapper<TaCarRecord> query = new QueryWrapper<>();
        query.ge("create_date", fourHour);
        query.groupBy("plate_number");
        query.orderByDesc("create_date");
        List<TaCarRecord> result = taCarRecordMapper.selectList(query);

        return result;
    }
}
